//
// This file is part of Nolimips, a MIPS simulator with unlimited registers
// Copyright (C) 2003, 2004, 2006 Benoit Perrot <benoit@lrde.epita.fr>
//
// Nolimips is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Nolimips is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//
/** \file vm/cp0.hh
    \brief Declare Coprocessor 0 class. */
#ifndef VM_CP0_HH
# define VM_CP0_HH

# include "vm/fwd.hh"

namespace inst { class Register; }

namespace vm
{

  /// Coprocessor 0 (control coprocessor) abstraction
  class Cp0
  {
  public:
    typedef uint32_t	register_type;

  public:
    /// Coprocessor 0 registers indices
    enum kind_type
      {
	index = 0,		///< Index register
	random = 1,		///< Random register
	entry_lo0 = 2,		///< Entry for even pages
	entry_lo1 = 3,		///< Entry for odd pages
	context = 4,		///< Context register
	page_mask = 5,		///< Page mask register
	wired = 6,		///< Wired register
	error = 7,		///< Status/control register
	bad_vaddr = 8,		///< Bad virtual address register
	count = 9,		///< Count register
	entry_hi = 10,		///< Entry high
	compare = 11,		///< Compare register
	status = 12,		///< Status (kernel/user mode, etc.) register
	cause = 13,		///< Cause of exception register
	epc = 14,		///< Exception program counter
	pr_id = 15,		///< Processor identification
	config = 16,		///< Configuration register (64 bits)
	lladdr = 17,		///< Loaded linked address
	watch_lo = 18,		///< Watch point debug facility, low part
	watch_hi = 19,		///< Watch point debug facility, high part
	x_context = 20,		///< X context register
	reserved0 = 21,		///< (Reserved register)
	reserved1 = 22,		///< (Reserved register)
	debug = 23,		///< Debug register
	depc = 24,		///< Debug exception program counter
	perf_cnt = 25,		///< Performance counter register
	err_ctl = 26,		///< Error control register
	cache_err = 27,		///< Cache error register
	cache_tag_lo = 28,	///< Cache tag/data low
	cache_tag_hi = 29,	///< Cache tag/data high
	error_epc = 30,		///< Error exception program counter
	desave = 31		///< Debug exception SAVE
      };

  public:
    /// Cause exception codes
    enum exception_type
      {
	interrupt = 0,		///< Interrupt
	tlb_modified = 1,	///< TLB modified
	tlb_load = 2,		///< TLB load
	tlb_store = 3,		///< TLB store
	addr_load = 4,		///< Address error on load
	addr_store = 5,		///< Address error on store
	bus_inst = 6,		///< Bus error on instruction fetch
	bus_data = 7,		///< Bus error on data load or store
	syscall = 8,		///< System call
	breakpoint = 9,		///< Breakpoint
	reserved_inst = 10,	///< Reserved instruction
	cop_unusable = 11,	///< Coprocessor unusable
	overflow = 12,		///< Arithmetic overflow
	trap = 13,		///< Trap
	floating_point = 15	///< Floating point exception
      };

    /** \name Constructor and destructor
	\{ */
  public:
    Cp0(Cpu &cpu);
    /** \} */

  public:
    /// Reset the CPU (set registers to zero)
    void reset();

    void set_system_library(SystemLibrary *l);

    /** \name Explicit register accessors
	\{ */
  public:
    void set_count(register_type r);
    register_type get_count() const;
    /** \} */

    /** \name Unified register accessors
	\{ */
  public:
    /// Set the register identified by \a reg to \a r.
    void set_register(const inst::Register &reg, register_type r);
    /// Return the register identified by \a reg.
    register_type get_register(const inst::Register &reg) const;

    register_type get_register(kind_type k) const;
    /** \} */


    /** \name Exception handling
        \{ */
  public:
    /// Raise an arithmetic overflow exception
    void raise_overflow();

    /// Raise an address error on load exception
    void raise_addr_load();
    /// Raise an address error on load exception
    void raise_addr_store();

    void raise_syscall();

    void set_fatal_exception();
    bool fatal_exception() const;
    /** \} */


  protected:
    Cpu &cpu_;

    bool fatal_exception_;

    /// Dedicated registers (32 bits)
    register_type registers[32];

    SystemLibrary *system_library_;
  };

} // namespace vm

#include "vm/cp0.hxx"

#endif // !VM_CP0_HH
